def format_readings(feast):
    readings = feast["readings"]["readings"]
    reading_dict = {}
    for reading in readings:
        reading_dict[reading["readingType"]] = reading["acronyme"]
    return f"{feast['annualText']}:\n" + "\n".join(reading_dict.values())


def format_heading(feast):
    headings = (f"Tema: {feast['feastHeading']}", feast["otherName"])
    return "\n".join(filter(bool, headings))


def format_altar(feast):
    return (
        f"Den liturgiska färgen är {feast['liturgicalColorDisplay'].lower()}. "
        f"{feast['altar']['fullText']}."
    )


def format_church_year(feast):
    return "\n".join(
        (feast["churchYearPart"]["name"], feast["churchYearPart"]["description"])
    )


def format_description(feast):
    components = (
        format_heading(feast),
        feast["feastText"],
        format_readings(feast),
        format_altar(feast),
        format_church_year(feast),
    )
    return "\n\n".join(filter(bool, components))



import json
import sys
import tempfile
from http.client import responses
from pathlib import Path

import httpx
import rich_click as click

from kyrkokalender import church_calendar

API_URL = "https://www.svenskakyrkan.se/webapi/api-v2/churchcalendar/{}?apiKey={}"


@click.command()
@click.argument("year", type=int, help="The year (counted from Advent Sunday) to create.")
@click.option("-r", "--reload", is_flag=True, help="Force new download of calender data.")
@click.option("--api-key", type=str, default=None, help="The API key to use.")
@click.option(
    "--api-key-path",
    type=click.Path(path_type=Path),
    default=Path(".api_key"),
    help="File containing The API key to use.",
)
def cli(year: int, reload: bool, api_key: str | None, api_key_path: Path) -> None:
    tmp_dir = Path(tempfile.gettempdir())
    cached_response_path = tmp_dir / f"kyrkokalender/{year}_response.json"
    if not cached_response_path.exists() or reload:
        if api_key or (api_key_path and api_key_path.exists()):
            api_key = api_key or api_key_path.read_text()
        else:
            sys.exit(
                "API key not provided. Add it with --api-key or --api-key-path. If your"
                "current directory has a file named .api_key, it will be used"
                "automatically."
            )

        url = API_URL.format(year, api_key)
        print("Fetching calendar data...")
        try:
            api_response = httpx.get(url)
            api_response.raise_for_status()
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            print(f"Error from server: {status_code} ({responses[status_code]})")
            match status_code:
                case 404 | 416:
                    print("Check year.")
                case 401:
                    print("Check API key.")
                case _:
                    print("Unknown error")
            sys.exit()

        calendar_data = api_response.json()
        cached_response_path.parent.mkdir(parents=True, exist_ok=True)
        with cached_response_path.open("w") as cache_file:
            json.dump(calendar_data, cache_file, indent=4)
    else:
        print("Opening cached response...")
        calendar_data = json.loads(cached_response_path.read_text())

    calendar = church_calendar.calendar_for_feasts(year, calendar_data)
    ics_path = Path(f"kyrkoåret_{year}.ics")
    print(f"Calendar created with {len(calendar.events)} feasts.")
    print(f"Writing ICS file to {ics_path.absolute()}")
    ics_path.write_bytes(calendar.to_ical())


if __name__ == "__main__":
    cli()



from datetime import datetime

import pytz
from icalendar import Calendar, Event

from kyrkokalender import feast_formatters


def event_for_feast(year: int, feast: dict) -> Event:
    feast_id = feast["id"]
    date = datetime.fromisoformat(feast["startDate"]).date()
    event = Event()
    event.add("summary", feast["feastName"])
    event.add("description", feast_formatters.format_description(feast))
    event.add("dtstart", date)
    event.add("dtstamp", datetime.now().replace(tzinfo=pytz.timezone("Europe/Stockholm")))
    event.add("uid", f"feast-{year}-{feast_id}@svenskakyrkan.se")
    event.add(
        "url",
        f"https://www.svenskakyrkan.se/kyrkoaret/bibeltexter?id={feast_id}&year={year}",
    )
    return event


def create_calendar() -> Calendar:
    calendar = Calendar()
    calendar.add("prodid", "+//IDN skogsfors.net")
    calendar.add("version", "2.0")
    return calendar


def calendar_for_feasts(year: int, feasts: dict) -> Calendar:
    calendar = create_calendar()
    for feast in feasts:
        event = event_for_feast(year, feast)
        calendar.add_component(event)
    return calendar



from pathlib import Path

import icalendar
import rich_click as click
from rich.console import Console
from rich.table import Table


@click.command()
@click.argument(
    "calendar_path",
    type=click.Path(exists=True, path_type=Path),
    help="The path to the calendar file.",
)
def cli(calendar_path: Path) -> None:
    calendar = icalendar.Calendar.from_ical(calendar_path.read_bytes())

    summary_table = Table(show_header=False)

    summary_table.add_column("Key", justify="left", style="cyan", no_wrap=True)
    summary_table.add_column("Value", style="magenta")
    summary_table.add_row("Feasts", str(len(calendar.events)))
    first_feast = calendar.events[0]
    summary_table.add_row(
        "First", f"{first_feast['summary']} ({first_feast['dtstart'].dt})"
    )
    last_feast = calendar.events[-1]
    summary_table.add_row("Last", f"{last_feast['summary']} ({last_feast['dtstart'].dt})")

    console = Console()
    console.print(summary_table)

    full_table = Table()
    full_table.add_column("Sunday", justify="left", style="cyan", no_wrap=True)
    full_table.add_column("Date", justify="left", style="magenta", no_wrap=True)

    current_month = first_feast["dtstart"].dt.month
    for feast in calendar.events:
        if current_month != feast["dtstart"].dt.month:
            current_month = feast["dtstart"].dt.month
            full_table.add_section()
        full_table.add_row(feast["summary"], f"{feast['dtstart'].dt}")

    console.print(full_table)


if __name__ == "__main__":
    cli()
